<?php
namespace Drupal\bg_rotate\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;

/**
 * Image rotate file upload form.
 */
class BgRotateSettings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bg_rotate_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function getEditableConfigNames() {
    return [
      'bg_rotate.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Get config.
    $config = $this->config('bg_rotate.settings');
    // Multiple file upload.
    $form['images'] = [
      '#type' => 'managed_file',
      '#title' => t('Files'),
      '#description' => t('Select background images.'),
      '#default_value' => $config->get('images'),
      '#multiple' => TRUE,
      '#required' => FALSE,
      '#progress_indicator' => 'bar',
      '#upload_location' => 'public://images/bg_rotate/',
      '#upload_validators' => [
        'file_validate_extensions' => ['png jpg jpeg gif'],
        'file_validate_size' => array(10000000),
      ],
    ];

    $form['selector'] = [
      '#type' => 'textfield',
      '#title' => t('CSS selector'),
      '#description' => t('Enter css selector here, default will be body'),
      '#default_value' => $config->get('selector') ? $config->get('selector') : 'body',
    ];

    $form['interval'] = [
      '#type' => 'select',
      '#title' => t('Time interval'),
      '#description' => t('Time between background switching.'),
      '#options' => [
        'month' => t('Month'),
        'week' => t('Week'),
        'day' => t('Day'),
        'hour' => t('Hour'),
        'minute' => t('Minute'),
        'second' => t('Second'),
      ],
      '#default_value' => $config->get('interval'),
    ];

    // Image sizes.
    $form['image_styles'] = [
      '#type' => 'fieldset',
      '#title' => t('Image styles'),
    ];

    $form['image_styles']['screen'] = [
      '#type' => 'select',
      '#title' => t('Screen'),
      '#options' => image_style_options(FALSE),
      '#default_value' => $config->get('screen'),
    ];

    $form['image_styles']['tablet'] = [
      '#type' => 'select',
      '#title' => t('Tablet'),
      '#options' => image_style_options(FALSE),
      '#default_value' => $config->get('tablet'),
    ];

    $form['image_styles']['phone'] = [
      '#type' => 'select',
      '#title' => t('Phone'),
      '#options' => image_style_options(FALSE),
      '#default_value' => $config->get('phone'),
    ];

    // Submit.
    $form['actions']['#type'] = 'container';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => t('Save'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('bg_rotate.settings');
    $file_usage = \Drupal::service('file.usage');

    // Check removed files and delete.
    if ($config->get('images')) {
      $diff = array_diff($config->get('images'), $form_state->getValue('images'));
      if (!empty($diff)) {
        $files = file_load_multiple($diff);
        foreach ($files as $file) {
          $file_usage->delete($file, 'bg_rotate', 'file', $file->id());
          $file->delete();
        }
      }
    }

    // Save settings.
    $this->config('bg_rotate.settings')
      ->set('images', $form_state->getValue('images'))
      ->set('selector', $form_state->getValue('selector'))
      ->set('interval', $form_state->getValue('interval'))
      ->set('screen', $form_state->getValue('screen'))
      ->set('tablet', $form_state->getValue('tablet'))
      ->set('phone', $form_state->getValue('phone'))
      ->save();

    // Make files permanent.
    $files = file_load_multiple($config->get('images'));
    foreach ($files as $file) {
      // Check if file has usage and add if needed.
      if (!$file_usage->listUsage($file)) {
        $file_usage->add($file, 'bg_rotate', 'file', $file->id());
        $list = $file_usage->listUsage($file);
      }
    }
    drupal_set_message(t('Settings saved.'));
  }

}
